@@ -3,7 +3,7 @@
 """
 Editor classes.
 
-Copyright (C) 2005, 2006 Paul Boddie <paul@boddie.org.uk>
+Copyright (C) 2005, 2006, 2007 Paul Boddie <paul@boddie.org.uk>
 
 This software is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License as
@@ -37,7 +37,6 @@ class ItemEditorResource(XSLForms.Resources.WebResources.XSLFormsResource):
     "An event/to-do item editor."
 
     resource_dir = os.path.join(os.path.split(__file__)[0], "Resources")
-    encoding = "utf-8"
     template_resources = {
         "event" : ("event_template.xhtml", "event_output.xsl"),
         "to-do" : ("to_do_template.xhtml", "to_do_output.xsl"),
@@ -62,6 +61,9 @@ class ItemEditorResource(XSLForms.Resources.WebResources.XSLFormsResource):
         "journal-datetimes" : ("journal", "journal_datetimes_output.xsl", "datetimes-node")
     }
     document_resources = {
+        "hour" : "event_hours.xml",
+        "minute" : "event_minutes_seconds.xml",
+        "second" : "event_minutes_seconds.xml",
         "adr" : "card_adr.xml",
         "label" : "card_label.xml",
         "tel" : "card_tel.xml",
@@ -80,8 +82,8 @@ class ItemEditorResource(XSLForms.Resources.WebResources.XSLFormsResource):
 
         # Get useful information about the request.
 
-        pvpath = trans.get_processed_virtual_path_info(self.path_encoding)
-        parent_path = trans.get_path_without_info(self.path_encoding) + trans.update_path(pvpath, "")
+        pvpath = trans.get_processed_virtual_path_info()
+        parent_path = trans.get_path_without_info() + trans.update_path(pvpath, "")
 
         # Determine which item is being edited.
         # The item type can be advisory since this may be a new item where we
@@ -117,11 +119,11 @@ class ItemEditorResource(XSLForms.Resources.WebResources.XSLFormsResource):
             self._add_dates(doc, item)
 
             if self.save_document(doc, item, item_type):
-                trans.redirect(trans.encode_path(parent_path, self.path_encoding))
+                trans.redirect(trans.encode_path(parent_path))
                 # Response ended!
 
         elif parameters.has_key("cancel"):
-            trans.redirect(trans.encode_path(parent_path, self.path_encoding))
+            trans.redirect(trans.encode_path(parent_path))
             # Response ended!
 
         # (Card selectors)
@@ -155,6 +157,21 @@ class ItemEditorResource(XSLForms.Resources.WebResources.XSLFormsResource):
                 "label" : self.prepare_document("label"),
                 "tel" : self.prepare_document("tel")
                 })
+
+        elif item_type == "event":
+
+            # Produce datetime type enumerations.
+
+            self._make_hours(doc)
+            self._make_minutes(doc)
+            self._make_seconds(doc)
+
+            doc = self.get_result([init_xsl], doc, stylesheet_expressions={
+                "hour" : "/item",
+                "minute" : "/item",
+                "second" : "/item"
+                })
+
         else:
             doc = self.get_result([init_xsl], doc)
 
@@ -179,8 +196,8 @@ class ItemEditorResource(XSLForms.Resources.WebResources.XSLFormsResource):
 
         # Produce calendars for the start and end of the event.
 
-        dtstart, (dtstart_date, dtstart_time) = self._get_dtelement(doc, "/item/*/dtstart")
-        dtend, (dtend_date, dtend_time) = self._get_dtelement(doc, "/item/*/dtend")
+        dtstart, dtstart_date = self._get_dtelement(doc, "/item/*/dtstart")
+        dtend, dtend_date = self._get_dtelement(doc, "/item/*/dtend")
         dtstart_month, start_date = self._get_month(doc, "/item/*/dtstart/month")
         dtend_month, end_date = self._get_month(doc, "/item/*/dtend/month")
 
@@ -207,16 +224,16 @@ class ItemEditorResource(XSLForms.Resources.WebResources.XSLFormsResource):
 
         if parameters.has_key("previous-start"):
             start_date, _tmp = self._previous_month(start_date)
-            dtstart_date, dtstart_time = None, None
+            dtstart_date = None
         elif parameters.has_key("next-start"):
             start_date, end_date = self._next_month(start_date, end_date)
-            dtstart_date, dtstart_time = None, None
+            dtstart_date = None
         elif parameters.has_key("previous-end"):
             end_date, start_date = self._previous_month(end_date, start_date)
-            dtend_date, dtend_time = None, None
+            dtend_date = None
         elif parameters.has_key("next-end"):
             end_date, _tmp = self._next_month(end_date)
-            dtend_date, dtend_time = None, None
+            dtend_date = None
 
         # Where no month navigation values are present, use the current date.
 
@@ -233,9 +250,9 @@ class ItemEditorResource(XSLForms.Resources.WebResources.XSLFormsResource):
             if dtstart_month is not None:
                 dtstart_month.parentNode.removeChild(dtstart_month)
             if dtstart_date is not None:
-                dtstart.setAttribute("datetime", "%04d%02d%02d%s" % (dtstart_date + (dtstart_time,)))
-            elif dtstart.hasAttribute("datetime"):
-                dtstart.removeAttribute("datetime")
+                dtstart.setAttribute("date", "%04d%02d%02d" % dtstart_date)
+            elif dtstart.hasAttribute("date"):
+                dtstart.removeAttribute("date")
             XMLCalendar.write_month_to_document(doc, dtstart, start_date[0], start_date[1])
 
             # Add the current date for informational purposes.
@@ -246,9 +263,9 @@ class ItemEditorResource(XSLForms.Resources.WebResources.XSLFormsResource):
             if dtend_month is not None:
                 dtend_month.parentNode.removeChild(dtend_month)
             if dtend_date is not None:
-                dtend.setAttribute("datetime", "%04d%02d%02d%s" % (dtend_date + (dtend_time,)))
-            elif dtend.hasAttribute("datetime"):
-                dtend.removeAttribute("datetime")
+                dtend.setAttribute("date", "%04d%02d%02d" % dtend_date)
+            elif dtend.hasAttribute("date"):
+                dtend.removeAttribute("date")
             XMLCalendar.write_month_to_document(doc, dtend, end_date[0], end_date[1])
 
             # Add the current date for informational purposes.
@@ -263,7 +280,7 @@ class ItemEditorResource(XSLForms.Resources.WebResources.XSLFormsResource):
 
         # Start the response.
 
-        trans.set_content_type(ContentType("application/xhtml+xml", self.encoding))
+        trans.set_content_type(ContentType("application/xhtml+xml"))
 
         # Complete the response.
 
@@ -332,12 +349,12 @@ class ItemEditorResource(XSLForms.Resources.WebResources.XSLFormsResource):
                 else:
                     prepare_xsl_list = self.prepare_transform("card")
 
+                # Convert the end dates to the form expected by the template.
+
+                self._fix_end_dates(doc, -1)
+
                 doc = self.get_result(prepare_xsl_list, doc)
 
-            # Convert the end dates to the form expected by the template.
-
-            self._fix_end_dates(doc, -1)
-
         else:
             doc = None
 
@@ -361,49 +378,61 @@ class ItemEditorResource(XSLForms.Resources.WebResources.XSLFormsResource):
             unprepare_xsl_list = self.prepare_transform("event-unprepare")
             doc = self.get_result(unprepare_xsl_list, doc)
 
-        # Remove any existing item.
+        # Attempt to maintain consistency in the store.
 
-        if item is not None:
-            self.store.remove_item(item)
+        try:
 
-        # Where no existing item exists, try and find a reasonable identifier.
-        # Do not allow items to be saved without a proper item reference.
+            # Remove any existing item.
 
-        elif item_type == "card":
-            uriref_values = doc.xpath("/item/*//email/@details")
-            if len(uriref_values) > 0:
-                uriref_value = uriref_values[0].nodeValue
-                if uriref_value:
-                    item = self.store.URIRef("MAILTO:" + uriref_value)
+            if item is not None:
+                self.store.remove_item(item)
+
+            # Where no existing item exists, try and find a reasonable identifier.
+            # Do not allow items to be saved without a proper item reference.
+
+            elif item_type == "card":
+                uriref_values = doc.xpath("/item/*//email/@details")
+                if len(uriref_values) > 0:
+                    uriref_value = uriref_values[0].nodeValue
+                    if uriref_value:
+                        item = self.store.URIRef("MAILTO:" + uriref_value)
+                    else:
+                        return 0
                 else:
                     return 0
-            else:
-                return 0
 
-        elif item_type in ("event", "to-do", "journal"):
-            summary_values = doc.xpath("/item/*/summary/@details")
-            if len(summary_values) > 0:
-                summary_value = summary_values[0].nodeValue
-                if summary_value:
-                    # NOTE: This will not work with various leading characters.
-                    # Change the summary into a URL-encoded identifier.
-                    item = self.store.get_item_from_identifier(urllib.quote(summary_value).replace("/", "%2F").replace("#", "%23"))
-                    while self.store.has_item(item):
-                        summary_value += "'"
+            elif item_type in ("event", "to-do", "journal"):
+                summary_values = doc.xpath("/item/*/summary/@details")
+                if len(summary_values) > 0:
+                    summary_value = summary_values[0].nodeValue
+                    if summary_value:
+                        # NOTE: This will not work with various leading characters.
+                        # Change the summary into a URL-encoded identifier.
                         item = self.store.get_item_from_identifier(urllib.quote(summary_value).replace("/", "%2F").replace("#", "%23"))
+                        while self.store.has_item(item):
+                            summary_value += "'"
+                            item = self.store.get_item_from_identifier(urllib.quote(summary_value).replace("/", "%2F").replace("#", "%23"))
+                    else:
+                        return 0
                 else:
                     return 0
-            else:
-                return 0
 
-        # Convert the end dates from the form expected by the template.
+            # Convert the end dates from the form expected by the template.
 
-        self._fix_end_dates(doc, 1)
+            self._fix_end_dates(doc, 1)
 
-        # Add a new item.
+            # Add a new item.
 
-        self.store.add_item(item, item_type, doc)
-        return 1
+            self.store.add_item(item, item_type, doc)
+            self.store.commit()
+            return 1
+
+        # If anything at all goes wrong, roll the transaction back and raise the
+        # exception.
+
+        except:
+            self.store.rollback()
+            raise
 
     def _fix_end_dates(self, doc, days):
 
@@ -457,14 +486,17 @@ class ItemEditorResource(XSLForms.Resources.WebResources.XSLFormsResource):
         dtelements = doc.xpath(name_path)
         if dtelements:
             dtelement = dtelements[0]
-            if dtelement.hasAttribute("datetime"):
-                dtelement_value = dtelement.getAttribute("datetime")
-                dtelement_year, dtelement_month, dtelement_day, dtelement_time = \
-                    int(dtelement_value[:4]), int(dtelement_value[4:6]), int(dtelement_value[6:8]), dtelement_value[8:]
-                return dtelement, ((dtelement_year, dtelement_month, dtelement_day), dtelement_time)
+            if dtelement.hasAttribute("date"):
+                dtelement_value = dtelement.getAttribute("date")
+                try:
+                    dtelement_year, dtelement_month, dtelement_day = \
+                        int(dtelement_value[:4]), int(dtelement_value[4:6]), int(dtelement_value[6:8])
+                    return dtelement, (dtelement_year, dtelement_month, dtelement_day)
+                except ValueError:
+                    return dtelement, None
             else:
-                return dtelement, (None, None)
-        return None, (None, None)
+                return dtelement, None
+        return None, None
 
     def _get_month(self, doc, name_path):
         months = doc.xpath(name_path)
@@ -501,6 +533,26 @@ class ItemEditorResource(XSLForms.Resources.WebResources.XSLFormsResource):
             limit_date = year, month
         return (year, month), limit_date
 
+    # Dynamic generation of enumerations.
+
+    def _make_hours(self, doc):
+        self._make_enum(doc, "hour", "hour-enum", 24)
+
+    def _make_minutes(self, doc):
+        self._make_enum(doc, "minute", "minute-enum", 60)
+
+    def _make_seconds(self, doc):
+        self._make_enum(doc, "second", "second-enum", 60)
+
+    def _make_enum(self, doc, name, name_enum, limit):
+        root = doc.xpath("*")[0]
+        top = doc.createElement(name)
+        root.appendChild(top)
+        for i in range(0, limit):
+            enum = doc.createElement(name_enum)
+            enum.setAttribute("value", "%02d" % i)
+            top.appendChild(enum)
+
     # Person initialisation methods.
 
     def _insert_cards(self, doc, root, element_name="cards"):
