@@ -89,7 +89,7 @@ class CalendarViewerResource(XSLForms.Resources.WebResources.XSLFormsResource):
         if parameters.has_key("download-all"):
             trans.set_content_type(ContentType("text/calendar"))
             stream = trans.get_response_stream()
-            export_items(found_items, self.store, stream)
+            export_items(found_items, self.store, stream, 1)
             raise EndOfResponse
 
         elif parameters.has_key("new"):
@@ -154,7 +154,8 @@ class CalendarViewerResource(XSLForms.Resources.WebResources.XSLFormsResource):
             language = ""
         stylesheet_parameters["locale"] = language
         stylesheet_parameters["root"] = attributes["root"]
-
+        stylesheet_parameters["metadata"] = parameters.get("metadata", ["false"])[0]
+            
         # Complete the response.
 
         self.send_output(trans, [trans_xsl], items, stylesheet_parameters,
@@ -239,13 +240,13 @@ class ListViewerResource(XSLForms.Resources.WebResources.XSLFormsResource):
         if parameters.has_key("download-all"):
             trans.set_content_type(ContentType("text/calendar"))
             stream = trans.get_response_stream()
-            export_items(found_items, self.store, stream)
+            export_items(found_items, self.store, stream, 1)
             raise EndOfResponse
 
         elif parameters.has_key("download"):
             trans.set_content_type(ContentType("text/calendar"))
             stream = trans.get_response_stream()
-            export_items(selected_items, self.store, stream)
+            export_items(selected_items, self.store, stream, 1)
             raise EndOfResponse
 
         elif parameters.has_key("new"):
