@@ -41,7 +41,8 @@ icon_sets = {
         ("apps", "kwrite2", "updated"),
         ("apps", "clock", "appointment"),
         ("actions", "redo", "continued"),
-        ("filesystems", "folder", None)
+        ("filesystems", "folder", None),
+        ("filesystems", "exec", "details")
         ],
     CRYSTAL_CLEAR : [
         ("mimetypes", "vcard", "card"),
@@ -58,7 +59,8 @@ icon_sets = {
         ("actions", "edit", "updated"),
         ("apps", "xclock", "appointment"),
         ("actions", "redo", "continued"),
-        ("filesystems", "folder", None)
+        ("filesystems", "folder", None),
+        ("actions", "exec", "details")
         ],
     TANGO : [
         #("apps", "system-users", "card"),
@@ -75,7 +77,8 @@ icon_sets = {
         ("actions", "document-open", "updated"),
         ("actions", "appointment", None),
         ("actions", "go-jump", "continued"),
-        ("places", "folder", None)
+        ("places", "folder", None),
+        ("actions", "list-add", "details")
         ]
     }
 
@@ -204,7 +207,10 @@ if __name__ == "__main__":
             # NOTE: equivalent).
             if args.has_key("convert"):
                 cmd = "ksvgtopng %s %s %s %s" % (width, height, image, target_image)
-                os.system(cmd)
+                # Recover by trying .svgz files.
+                if not os.system(cmd):
+                    cmd = "ksvgtopng %s %s %sz %s" % (width, height, image, target_image)
+                    os.system(cmd)
             else:
                 shutil.copy(image, target_image)
             f.write("Image %s copied to produce %s.\n" % (icon_filename, new_icon_filename))
